

package org.apache.drill.exec.expr.fn.impl;

import org.apache.drill.exec.expr.DrillSimpleFunc;
import org.apache.drill.exec.expr.annotations.FunctionTemplate;
import org.apache.drill.exec.expr.annotations.FunctionTemplate.NullHandling;
import org.apache.drill.exec.expr.annotations.Output;
import org.apache.drill.exec.expr.annotations.Param;
import org.apache.drill.exec.expr.fn.FunctionGenerationHelper;
import org.apache.drill.exec.expr.fn.impl.ByteFunctionHelpers;
import org.apache.drill.exec.expr.holders.*;
import org.apache.drill.exec.record.RecordBatch;
import javax.inject.Inject;
import io.netty.buffer.DrillBuf;

/*
 * This class is generated using freemarker and the ComparisonFunctions.java template.
 */

@SuppressWarnings("unused")
public class GCompareIntervalDayVsIntervalDay {



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareIntervalDayVsIntervalDayNullHigh implements DrillSimpleFunc {

    @Param IntervalDayHolder left;
    @Param IntervalDayHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {


    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        out.value = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareIntervalDayVsIntervalDayNullLow implements DrillSimpleFunc {

    @Param IntervalDayHolder left;
    @Param IntervalDayHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {


    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        out.value = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
    }
  }



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareIntervalDayVsNullableIntervalDayNullHigh implements DrillSimpleFunc {

    @Param IntervalDayHolder left;
    @Param NullableIntervalDayHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

      if ( right.isSet == 0 ) {
        out.value = -1;
        break outside;
      }

    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        out.value = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareIntervalDayVsNullableIntervalDayNullLow implements DrillSimpleFunc {

    @Param IntervalDayHolder left;
    @Param NullableIntervalDayHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

      if ( right.isSet == 0 ) {
        out.value = 1;
        break outside;
      }

    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        out.value = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
    }
  }



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableIntervalDayVsIntervalDayNullHigh implements DrillSimpleFunc {

    @Param NullableIntervalDayHolder left;
    @Param IntervalDayHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          out.value = 1;
          break outside;
        }

    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        out.value = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableIntervalDayVsIntervalDayNullLow implements DrillSimpleFunc {

    @Param NullableIntervalDayHolder left;
    @Param IntervalDayHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          out.value = -1;
          break outside;
        }

    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        out.value = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
    }
  }



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableIntervalDayVsNullableIntervalDayNullHigh implements DrillSimpleFunc {

    @Param NullableIntervalDayHolder left;
    @Param NullableIntervalDayHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          if ( right.isSet == 0 ) {
            out.value = 0;
            break outside;
          } else {
            out.value = 1;
            break outside;
          }
        } else if ( right.isSet == 0 ) {
          out.value = -1;
          break outside;
        }

    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        out.value = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableIntervalDayVsNullableIntervalDayNullLow implements DrillSimpleFunc {

    @Param NullableIntervalDayHolder left;
    @Param NullableIntervalDayHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          if ( right.isSet == 0 ) {
            out.value = 0;
            break outside;
          } else {
            out.value = -1;
            break outside;
          }
        } else if ( right.isSet == 0 ) {
          out.value = 1;
          break outside;
        }

    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        out.value = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
    }
  }

   @FunctionTemplate(names = {"less_than", "<"},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class LessThanIntervalDayVsIntervalDay implements DrillSimpleFunc {

      @Param IntervalDayHolder left;
      @Param IntervalDayHolder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

          int cmp;
     outside:
      {


    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        cmp = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
          out.value = cmp == -1 ? 1 : 0;

      }
  }

  @FunctionTemplate(names = {"less_than_or_equal_to", "<="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class LessThanEqIntervalDayVsIntervalDay implements DrillSimpleFunc {

      @Param IntervalDayHolder left;
      @Param IntervalDayHolder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

          int cmp;
     outside:
      {


    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        cmp = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
          out.value = cmp < 1 ? 1 : 0;

    }
  }

  @FunctionTemplate(names = {"greater_than", ">"},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class GreaterThanIntervalDayVsIntervalDay implements DrillSimpleFunc {

      @Param IntervalDayHolder left;
      @Param IntervalDayHolder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

          int cmp;
     outside:
      {


    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        cmp = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
          out.value = cmp == 1 ? 1 : 0;

    }
  }

  @FunctionTemplate(names = {"greater_than_or_equal_to", ">="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class GreaterThanEqIntervalDayVsIntervalDay implements DrillSimpleFunc {

      @Param IntervalDayHolder left;
      @Param IntervalDayHolder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

          int cmp;
     outside:
      {


    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        cmp = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
          out.value = cmp > -1 ? 1 : 0;

      }
  }

  @FunctionTemplate(names = {"equal", "==", "="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class EqualsIntervalDayVsIntervalDay implements DrillSimpleFunc {

      @Param IntervalDayHolder left;
      @Param IntervalDayHolder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

          int cmp;
     outside:
      {


    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        cmp = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
          out.value = cmp == 0 ? 1 : 0;

      }
  }

  @FunctionTemplate(names = {"not_equal", "<>", "!="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class NotEqualsIntervalDayVsIntervalDay implements DrillSimpleFunc {

      @Param IntervalDayHolder left;
      @Param IntervalDayHolder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

          int cmp;
     outside:
      {


    // NaN is the biggest possible value, and NaN == NaN

        org.joda.time.MutableDateTime leftDate  =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);
        org.joda.time.MutableDateTime rightDate =
            new org.joda.time.MutableDateTime(1970, 1, 1, 0, 0, 0, 0, org.joda.time.DateTimeZone.UTC);

        // Left and right date have the same starting point (epoch), add the interval period and compare the two
        leftDate.addMonths(0);
        leftDate.addDays(left.days);
        leftDate.add(left.milliseconds);

        rightDate.addMonths(0);
        rightDate.addDays(right.days);
        rightDate.add(right.milliseconds);

        long leftMS  = leftDate.getMillis();
        long rightMS = rightDate.getMillis();

        cmp = leftMS < rightMS ? -1 : (leftMS > rightMS ? 1 : 0);


      } // outside
          out.value = cmp == 0 ? 0 : 1;

      }
  }

}


 



