

package org.apache.drill.exec.expr.fn.impl;

import org.apache.drill.exec.expr.DrillSimpleFunc;
import org.apache.drill.exec.expr.annotations.FunctionTemplate;
import org.apache.drill.exec.expr.annotations.FunctionTemplate.NullHandling;
import org.apache.drill.exec.expr.annotations.Output;
import org.apache.drill.exec.expr.annotations.Param;
import org.apache.drill.exec.expr.fn.FunctionGenerationHelper;
import org.apache.drill.exec.expr.fn.impl.ByteFunctionHelpers;
import org.apache.drill.exec.expr.holders.*;
import org.apache.drill.exec.record.RecordBatch;
import javax.inject.Inject;
import io.netty.buffer.DrillBuf;

/*
 * This class is generated using freemarker and the ComparisonFunctions.java template.
 */

@SuppressWarnings("unused")
public class GCompareIntervalYearVsIntervalYear {



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareIntervalYearVsIntervalYearNullHigh implements DrillSimpleFunc {

    @Param IntervalYearHolder left;
    @Param IntervalYearHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {


    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareIntervalYearVsIntervalYearNullLow implements DrillSimpleFunc {

    @Param IntervalYearHolder left;
    @Param IntervalYearHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {


    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareIntervalYearVsNullableIntervalYearNullHigh implements DrillSimpleFunc {

    @Param IntervalYearHolder left;
    @Param NullableIntervalYearHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

      if ( right.isSet == 0 ) {
        out.value = -1;
        break outside;
      }

    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareIntervalYearVsNullableIntervalYearNullLow implements DrillSimpleFunc {

    @Param IntervalYearHolder left;
    @Param NullableIntervalYearHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

      if ( right.isSet == 0 ) {
        out.value = 1;
        break outside;
      }

    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableIntervalYearVsIntervalYearNullHigh implements DrillSimpleFunc {

    @Param NullableIntervalYearHolder left;
    @Param IntervalYearHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          out.value = 1;
          break outside;
        }

    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableIntervalYearVsIntervalYearNullLow implements DrillSimpleFunc {

    @Param NullableIntervalYearHolder left;
    @Param IntervalYearHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          out.value = -1;
          break outside;
        }

    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableIntervalYearVsNullableIntervalYearNullHigh implements DrillSimpleFunc {

    @Param NullableIntervalYearHolder left;
    @Param NullableIntervalYearHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          if ( right.isSet == 0 ) {
            out.value = 0;
            break outside;
          } else {
            out.value = 1;
            break outside;
          }
        } else if ( right.isSet == 0 ) {
          out.value = -1;
          break outside;
        }

    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableIntervalYearVsNullableIntervalYearNullLow implements DrillSimpleFunc {

    @Param NullableIntervalYearHolder left;
    @Param NullableIntervalYearHolder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          if ( right.isSet == 0 ) {
            out.value = 0;
            break outside;
          } else {
            out.value = -1;
            break outside;
          }
        } else if ( right.isSet == 0 ) {
          out.value = 1;
          break outside;
        }

    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }

   @FunctionTemplate(names = {"less_than", "<"},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class LessThanIntervalYearVsIntervalYear implements DrillSimpleFunc {

      @Param IntervalYearHolder left;
      @Param IntervalYearHolder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

        // NaN is the biggest possible value, and NaN == NaN
        if (Double.isNaN(left.value) || ( Double.isNaN(left.value) && Double.isNaN(right.value))) {
          out.value=0;
        } else if (Double.isNaN(right.value) && !Double.isNaN(left.value)) {
          out.value = 1;
        } else {
          out.value = left.value < right.value ? 1 : 0;
        }

      }
  }

  @FunctionTemplate(names = {"less_than_or_equal_to", "<="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class LessThanEqIntervalYearVsIntervalYear implements DrillSimpleFunc {

      @Param IntervalYearHolder left;
      @Param IntervalYearHolder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

        // NaN is the biggest possible value, and NaN == NaN
        if (Double.isNaN(right.value)){
          out.value = 1;
        } else if (!Double.isNaN(right.value) && Double.isNaN(left.value)) {
          out.value = 0;
        } else {
          out.value = left.value <= right.value ? 1 : 0;
        }

    }
  }

  @FunctionTemplate(names = {"greater_than", ">"},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class GreaterThanIntervalYearVsIntervalYear implements DrillSimpleFunc {

      @Param IntervalYearHolder left;
      @Param IntervalYearHolder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

        // NaN is the biggest possible value, and NaN == NaN
        if (Double.isNaN(right.value) || ( Double.isNaN(left.value) && Double.isNaN(right.value))) {
          out.value = 0;
        } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
          out.value = 1;
        } else {
          out.value = left.value > right.value ? 1 : 0;
        }

    }
  }

  @FunctionTemplate(names = {"greater_than_or_equal_to", ">="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class GreaterThanEqIntervalYearVsIntervalYear implements DrillSimpleFunc {

      @Param IntervalYearHolder left;
      @Param IntervalYearHolder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

        // NaN is the biggest possible value, and NaN == NaN
        if (Double.isNaN(left.value)){
          out.value=1;
        } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
          out.value = 0;
        } else {
          out.value = left.value >= right.value ? 1 : 0;
        }


      }
  }

  @FunctionTemplate(names = {"equal", "==", "="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class EqualsIntervalYearVsIntervalYear implements DrillSimpleFunc {

      @Param IntervalYearHolder left;
      @Param IntervalYearHolder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

        // NaN is the biggest possible value, and NaN == NaN
        if (Double.isNaN(left.value) && Double.isNaN(right.value)) {
          out.value = 1;
        } else {
          out.value = left.value == right.value ? 1 : 0;
        }

      }
  }

  @FunctionTemplate(names = {"not_equal", "<>", "!="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class NotEqualsIntervalYearVsIntervalYear implements DrillSimpleFunc {

      @Param IntervalYearHolder left;
      @Param IntervalYearHolder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

        // NaN is the biggest possible value, and NaN == NaN
        if (Double.isNaN(left.value) && Double.isNaN(right.value)) {
          out.value = 0;
        } else {
          out.value = left.value != right.value ? 1 : 0;
        }

      }
  }

}


 



