

package org.apache.drill.exec.expr.fn.impl.gaggr;

import org.apache.drill.exec.expr.DrillAggFunc;
import org.apache.drill.exec.expr.annotations.FunctionTemplate;
import org.apache.drill.exec.expr.annotations.FunctionTemplate.FunctionScope;
import org.apache.drill.exec.expr.annotations.Output;
import org.apache.drill.exec.expr.annotations.Param;
import org.apache.drill.exec.expr.annotations.Workspace;
import org.apache.drill.exec.expr.holders.*;

import javax.inject.Inject;
import io.netty.buffer.DrillBuf;

/*
 * This class is generated using freemarker and the SingleValueAgg.java template.
 */
@SuppressWarnings("unused")
public class SingleValueFunctions {

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class BitSingleValue implements DrillAggFunc {
    @Param BitHolder in;
    @Workspace BitHolder value;
    @Output NullableBitHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new BitHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new BitHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class TinyIntSingleValue implements DrillAggFunc {
    @Param TinyIntHolder in;
    @Workspace TinyIntHolder value;
    @Output NullableTinyIntHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new TinyIntHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new TinyIntHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableTinyIntSingleValue implements DrillAggFunc {
    @Param NullableTinyIntHolder in;
    @Workspace TinyIntHolder value;
    @Output NullableTinyIntHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new TinyIntHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new TinyIntHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class UInt1SingleValue implements DrillAggFunc {
    @Param UInt1Holder in;
    @Workspace UInt1Holder value;
    @Output NullableUInt1Holder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new UInt1Holder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new UInt1Holder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableUInt1SingleValue implements DrillAggFunc {
    @Param NullableUInt1Holder in;
    @Workspace UInt1Holder value;
    @Output NullableUInt1Holder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new UInt1Holder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new UInt1Holder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class UInt2SingleValue implements DrillAggFunc {
    @Param UInt2Holder in;
    @Workspace UInt2Holder value;
    @Output NullableUInt2Holder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new UInt2Holder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new UInt2Holder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableUInt2SingleValue implements DrillAggFunc {
    @Param NullableUInt2Holder in;
    @Workspace UInt2Holder value;
    @Output NullableUInt2Holder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new UInt2Holder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new UInt2Holder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class SmallIntSingleValue implements DrillAggFunc {
    @Param SmallIntHolder in;
    @Workspace SmallIntHolder value;
    @Output NullableSmallIntHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new SmallIntHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new SmallIntHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableSmallIntSingleValue implements DrillAggFunc {
    @Param NullableSmallIntHolder in;
    @Workspace SmallIntHolder value;
    @Output NullableSmallIntHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new SmallIntHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new SmallIntHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class UInt4SingleValue implements DrillAggFunc {
    @Param UInt4Holder in;
    @Workspace UInt4Holder value;
    @Output NullableUInt4Holder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new UInt4Holder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new UInt4Holder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableUInt4SingleValue implements DrillAggFunc {
    @Param NullableUInt4Holder in;
    @Workspace UInt4Holder value;
    @Output NullableUInt4Holder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new UInt4Holder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new UInt4Holder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class UInt8SingleValue implements DrillAggFunc {
    @Param UInt8Holder in;
    @Workspace UInt8Holder value;
    @Output NullableUInt8Holder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new UInt8Holder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new UInt8Holder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableUInt8SingleValue implements DrillAggFunc {
    @Param NullableUInt8Holder in;
    @Workspace UInt8Holder value;
    @Output NullableUInt8Holder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new UInt8Holder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new UInt8Holder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class IntSingleValue implements DrillAggFunc {
    @Param IntHolder in;
    @Workspace IntHolder value;
    @Output NullableIntHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new IntHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new IntHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class BigIntSingleValue implements DrillAggFunc {
    @Param BigIntHolder in;
    @Workspace BigIntHolder value;
    @Output NullableBigIntHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new BigIntHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new BigIntHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableBitSingleValue implements DrillAggFunc {
    @Param NullableBitHolder in;
    @Workspace BitHolder value;
    @Output NullableBitHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new BitHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new BitHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableIntSingleValue implements DrillAggFunc {
    @Param NullableIntHolder in;
    @Workspace IntHolder value;
    @Output NullableIntHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new IntHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new IntHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableBigIntSingleValue implements DrillAggFunc {
    @Param NullableBigIntHolder in;
    @Workspace BigIntHolder value;
    @Output NullableBigIntHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new BigIntHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new BigIntHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class Float4SingleValue implements DrillAggFunc {
    @Param Float4Holder in;
    @Workspace Float4Holder value;
    @Output NullableFloat4Holder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new Float4Holder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new Float4Holder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class Float8SingleValue implements DrillAggFunc {
    @Param Float8Holder in;
    @Workspace Float8Holder value;
    @Output NullableFloat8Holder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new Float8Holder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new Float8Holder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableFloat4SingleValue implements DrillAggFunc {
    @Param NullableFloat4Holder in;
    @Workspace Float4Holder value;
    @Output NullableFloat4Holder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new Float4Holder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new Float4Holder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableFloat8SingleValue implements DrillAggFunc {
    @Param NullableFloat8Holder in;
    @Workspace Float8Holder value;
    @Output NullableFloat8Holder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new Float8Holder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new Float8Holder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class DateSingleValue implements DrillAggFunc {
    @Param DateHolder in;
    @Workspace DateHolder value;
    @Output NullableDateHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new DateHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new DateHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableDateSingleValue implements DrillAggFunc {
    @Param NullableDateHolder in;
    @Workspace DateHolder value;
    @Output NullableDateHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new DateHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new DateHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class TimeStampSingleValue implements DrillAggFunc {
    @Param TimeStampHolder in;
    @Workspace TimeStampHolder value;
    @Output NullableTimeStampHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new TimeStampHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new TimeStampHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableTimeStampSingleValue implements DrillAggFunc {
    @Param NullableTimeStampHolder in;
    @Workspace TimeStampHolder value;
    @Output NullableTimeStampHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new TimeStampHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new TimeStampHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class TimeSingleValue implements DrillAggFunc {
    @Param TimeHolder in;
    @Workspace TimeHolder value;
    @Output NullableTimeHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new TimeHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new TimeHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableTimeSingleValue implements DrillAggFunc {
    @Param NullableTimeHolder in;
    @Workspace TimeHolder value;
    @Output NullableTimeHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new TimeHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new TimeHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class IntervalDaySingleValue implements DrillAggFunc {
    @Param IntervalDayHolder in;
    @Workspace IntervalDayHolder value;
    @Output NullableIntervalDayHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new IntervalDayHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.days = in.days;
      value.milliseconds = in.milliseconds;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.days = value.days;
        out.milliseconds = value.milliseconds;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new IntervalDayHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableIntervalDaySingleValue implements DrillAggFunc {
    @Param NullableIntervalDayHolder in;
    @Workspace IntervalDayHolder value;
    @Output NullableIntervalDayHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new IntervalDayHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.days = in.days;
      value.milliseconds = in.milliseconds;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.days = value.days;
        out.milliseconds = value.milliseconds;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new IntervalDayHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class IntervalYearSingleValue implements DrillAggFunc {
    @Param IntervalYearHolder in;
    @Workspace IntervalYearHolder value;
    @Output NullableIntervalYearHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new IntervalYearHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new IntervalYearHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableIntervalYearSingleValue implements DrillAggFunc {
    @Param NullableIntervalYearHolder in;
    @Workspace IntervalYearHolder value;
    @Output NullableIntervalYearHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new IntervalYearHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.value = in.value;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.value = value.value;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new IntervalYearHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class IntervalSingleValue implements DrillAggFunc {
    @Param IntervalHolder in;
    @Workspace IntervalHolder value;
    @Output NullableIntervalHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new IntervalHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.days = in.days;
      value.milliseconds = in.milliseconds;
      value.months = in.months;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.days = value.days;
        out.milliseconds = value.milliseconds;
        out.months = value.months;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new IntervalHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableIntervalSingleValue implements DrillAggFunc {
    @Param NullableIntervalHolder in;
    @Workspace IntervalHolder value;
    @Output NullableIntervalHolder out;
    @Workspace BigIntHolder nonNullCount;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new IntervalHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.days = in.days;
      value.milliseconds = in.milliseconds;
      value.months = in.months;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.days = value.days;
        out.milliseconds = value.milliseconds;
        out.months = value.months;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new IntervalHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    returnType = FunctionTemplate.ReturnType.DECIMAL_AVG_AGGREGATE,
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class VarDecimalSingleValue implements DrillAggFunc {
    @Param VarDecimalHolder in;
    @Workspace VarDecimalHolder value;
    @Output NullableVarDecimalHolder out;
    @Workspace BigIntHolder nonNullCount;
    @Inject DrillBuf buffer;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new VarDecimalHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.start = in.start;
      value.end = in.end;
      value.buffer = in.buffer;
      value.scale = in.scale;
      value.precision = in.precision;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.start = value.start;
        out.end = value.end;
        out.buffer = buffer.reallocIfNeeded(value.end - value.start);
        out.buffer.writeBytes(value.buffer, value.start, value.end - value.start);
        out.scale = value.scale;
        out.precision = value.precision;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new VarDecimalHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    returnType = FunctionTemplate.ReturnType.DECIMAL_AVG_AGGREGATE,
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableVarDecimalSingleValue implements DrillAggFunc {
    @Param NullableVarDecimalHolder in;
    @Workspace VarDecimalHolder value;
    @Output NullableVarDecimalHolder out;
    @Workspace BigIntHolder nonNullCount;
    @Inject DrillBuf buffer;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new VarDecimalHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.start = in.start;
      value.end = in.end;
      value.buffer = in.buffer;
      value.scale = in.scale;
      value.precision = in.precision;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.start = value.start;
        out.end = value.end;
        out.buffer = buffer.reallocIfNeeded(value.end - value.start);
        out.buffer.writeBytes(value.buffer, value.start, value.end - value.start);
        out.scale = value.scale;
        out.precision = value.precision;
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new VarDecimalHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class VarCharSingleValue implements DrillAggFunc {
    @Param VarCharHolder in;
    @Workspace VarCharHolder value;
    @Output NullableVarCharHolder out;
    @Workspace BigIntHolder nonNullCount;
    @Inject DrillBuf buffer;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new VarCharHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.start = in.start;
      value.end = in.end;
      value.buffer = in.buffer;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.start = value.start;
        out.end = value.end;
        out.buffer = buffer.reallocIfNeeded(value.end - value.start);
        out.buffer.writeBytes(value.buffer, value.start, value.end - value.start);
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new VarCharHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableVarCharSingleValue implements DrillAggFunc {
    @Param NullableVarCharHolder in;
    @Workspace VarCharHolder value;
    @Output NullableVarCharHolder out;
    @Workspace BigIntHolder nonNullCount;
    @Inject DrillBuf buffer;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new VarCharHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.start = in.start;
      value.end = in.end;
      value.buffer = in.buffer;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.start = value.start;
        out.end = value.end;
        out.buffer = buffer.reallocIfNeeded(value.end - value.start);
        out.buffer.writeBytes(value.buffer, value.start, value.end - value.start);
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new VarCharHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class Var16CharSingleValue implements DrillAggFunc {
    @Param Var16CharHolder in;
    @Workspace Var16CharHolder value;
    @Output NullableVar16CharHolder out;
    @Workspace BigIntHolder nonNullCount;
    @Inject DrillBuf buffer;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new Var16CharHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.start = in.start;
      value.end = in.end;
      value.buffer = in.buffer;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.start = value.start;
        out.end = value.end;
        out.buffer = buffer.reallocIfNeeded(value.end - value.start);
        out.buffer.writeBytes(value.buffer, value.start, value.end - value.start);
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new Var16CharHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableVar16CharSingleValue implements DrillAggFunc {
    @Param NullableVar16CharHolder in;
    @Workspace Var16CharHolder value;
    @Output NullableVar16CharHolder out;
    @Workspace BigIntHolder nonNullCount;
    @Inject DrillBuf buffer;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new Var16CharHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.start = in.start;
      value.end = in.end;
      value.buffer = in.buffer;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.start = value.start;
        out.end = value.end;
        out.buffer = buffer.reallocIfNeeded(value.end - value.start);
        out.buffer.writeBytes(value.buffer, value.start, value.end - value.start);
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new Var16CharHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class VarBinarySingleValue implements DrillAggFunc {
    @Param VarBinaryHolder in;
    @Workspace VarBinaryHolder value;
    @Output NullableVarBinaryHolder out;
    @Workspace BigIntHolder nonNullCount;
    @Inject DrillBuf buffer;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new VarBinaryHolder();
    }

    @Override
    public void add() {
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.start = in.start;
      value.end = in.end;
      value.buffer = in.buffer;
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.start = value.start;
        out.end = value.end;
        out.buffer = buffer.reallocIfNeeded(value.end - value.start);
        out.buffer.writeBytes(value.buffer, value.start, value.end - value.start);
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new VarBinaryHolder();
      nonNullCount.value = 0;
    }
  }

  @FunctionTemplate(name = "single_value",
                    scope = FunctionTemplate.FunctionScope.POINT_AGGREGATE)
  public static class NullableVarBinarySingleValue implements DrillAggFunc {
    @Param NullableVarBinaryHolder in;
    @Workspace VarBinaryHolder value;
    @Output NullableVarBinaryHolder out;
    @Workspace BigIntHolder nonNullCount;
    @Inject DrillBuf buffer;

    public void setup() {
      nonNullCount = new BigIntHolder();
      nonNullCount.value = 0;
      value = new VarBinaryHolder();
    }

    @Override
    public void add() {
      sout: {
        if (in.isSet == 0) {
          // processing nullable input and the value is null, so don't do anything...
          break sout;
        }
      if (nonNullCount.value == 0) {
        nonNullCount.value = 1;
      } else {
        throw org.apache.drill.common.exceptions.UserException.functionError()
            .message("Input for single_value function has more than one row")
            .build();
      }
      value.start = in.start;
      value.end = in.end;
      value.buffer = in.buffer;
      } // end of sout block
    }

    @Override
    public void output() {
      if (nonNullCount.value > 0) {
        out.isSet = 1;
        out.start = value.start;
        out.end = value.end;
        out.buffer = buffer.reallocIfNeeded(value.end - value.start);
        out.buffer.writeBytes(value.buffer, value.start, value.end - value.start);
      } else {
        out.isSet = 0;
      }
    }

    @Override
    public void reset() {
      value = new VarBinaryHolder();
      nonNullCount.value = 0;
    }
  }
}

