
package org.apache.drill.exec.store.parquet;

import org.apache.drill.common.types.TypeProtos.DataMode;
import org.apache.drill.common.types.TypeProtos.MinorType;
import org.apache.drill.exec.planner.types.DrillRelDataTypeSystem;
import org.apache.drill.exec.record.MaterializedField;
import org.apache.parquet.schema.OriginalType;
import org.apache.parquet.schema.DecimalMetadata;
import org.apache.parquet.schema.PrimitiveType.PrimitiveTypeName;
import org.apache.parquet.schema.Type.Repetition;

import java.util.HashMap;
import java.util.Map;

/*
 * This class is generated using freemarker and the ParquetTypeHelper.java template.
 */

public class ParquetTypeHelper {
  private static Map<MinorType,PrimitiveTypeName> typeMap;
  private static Map<DataMode,Repetition> modeMap;
  private static Map<MinorType,OriginalType> originalTypeMap;

  static {
    typeMap = new HashMap();

                    typeMap.put(MinorType.TINYINT, PrimitiveTypeName.INT32);
                    typeMap.put(MinorType.UINT1, PrimitiveTypeName.INT32);
                    typeMap.put(MinorType.UINT2, PrimitiveTypeName.INT32);
                    typeMap.put(MinorType.SMALLINT, PrimitiveTypeName.INT32);
                    typeMap.put(MinorType.INT, PrimitiveTypeName.INT32);
                    typeMap.put(MinorType.UINT4, PrimitiveTypeName.INT32);
                    typeMap.put(MinorType.FLOAT4, PrimitiveTypeName.FLOAT);
                    typeMap.put(MinorType.TIME, PrimitiveTypeName.INT32);
                    typeMap.put(MinorType.INTERVALYEAR, PrimitiveTypeName.FIXED_LEN_BYTE_ARRAY);
                    typeMap.put(MinorType.DECIMAL9, PrimitiveTypeName.INT32);
                    typeMap.put(MinorType.BIGINT, PrimitiveTypeName.INT64);
                    typeMap.put(MinorType.UINT8, PrimitiveTypeName.INT64);
                    typeMap.put(MinorType.FLOAT8, PrimitiveTypeName.DOUBLE);
                    typeMap.put(MinorType.DATE, PrimitiveTypeName.INT32);
                    typeMap.put(MinorType.TIMESTAMP, PrimitiveTypeName.INT64);
                    typeMap.put(MinorType.DECIMAL18, PrimitiveTypeName.INT64);
                    typeMap.put(MinorType.INTERVALDAY, PrimitiveTypeName.FIXED_LEN_BYTE_ARRAY);
                    typeMap.put(MinorType.INTERVAL, PrimitiveTypeName.FIXED_LEN_BYTE_ARRAY);
                    typeMap.put(MinorType.DECIMAL28DENSE, PrimitiveTypeName.FIXED_LEN_BYTE_ARRAY);
                    typeMap.put(MinorType.DECIMAL38DENSE, PrimitiveTypeName.FIXED_LEN_BYTE_ARRAY);
                    typeMap.put(MinorType.DECIMAL38SPARSE, PrimitiveTypeName.FIXED_LEN_BYTE_ARRAY);
                    typeMap.put(MinorType.DECIMAL28SPARSE, PrimitiveTypeName.FIXED_LEN_BYTE_ARRAY);
                    typeMap.put(MinorType.VARBINARY, PrimitiveTypeName.BINARY);
                    typeMap.put(MinorType.VARCHAR, PrimitiveTypeName.BINARY);
                    typeMap.put(MinorType.VAR16CHAR, PrimitiveTypeName.BINARY);
                    typeMap.put(MinorType.VARDECIMAL, PrimitiveTypeName.FIXED_LEN_BYTE_ARRAY);
                    typeMap.put(MinorType.BIT, PrimitiveTypeName.BOOLEAN);

    modeMap = new HashMap();

    modeMap.put(DataMode.REQUIRED, Repetition.REQUIRED);
    modeMap.put(DataMode.OPTIONAL, Repetition.OPTIONAL);
    modeMap.put(DataMode.REPEATED, Repetition.REPEATED);

    originalTypeMap = new HashMap();

            originalTypeMap.put(MinorType.DECIMAL9, OriginalType.DECIMAL);
            originalTypeMap.put(MinorType.DECIMAL18, OriginalType.DECIMAL);
            originalTypeMap.put(MinorType.DECIMAL28DENSE, OriginalType.DECIMAL);
            originalTypeMap.put(MinorType.DECIMAL38DENSE, OriginalType.DECIMAL);
            originalTypeMap.put(MinorType.DECIMAL38SPARSE, OriginalType.DECIMAL);
            originalTypeMap.put(MinorType.DECIMAL28SPARSE, OriginalType.DECIMAL);
            originalTypeMap.put(MinorType.VARDECIMAL, OriginalType.DECIMAL);
            originalTypeMap.put(MinorType.VARCHAR, OriginalType.UTF8);
            originalTypeMap.put(MinorType.DATE, OriginalType.DATE);
            originalTypeMap.put(MinorType.TIME, OriginalType.TIME_MILLIS);
            originalTypeMap.put(MinorType.TIMESTAMP, OriginalType.TIMESTAMP_MILLIS);
            originalTypeMap.put(MinorType.INTERVALDAY, OriginalType.INTERVAL);
            originalTypeMap.put(MinorType.INTERVALYEAR, OriginalType.INTERVAL);
            originalTypeMap.put(MinorType.INTERVAL, OriginalType.INTERVAL);
//            originalTypeMap.put(MinorType.TIMESTAMPTZ, OriginalType.TIMESTAMPTZ);
  }

  public static PrimitiveTypeName getPrimitiveTypeNameForMinorType(MinorType minorType) {
    return typeMap.get(minorType);
  }

  public static Repetition getRepetitionForDataMode(DataMode dataMode) {
    return modeMap.get(dataMode);
  }

  public static OriginalType getOriginalTypeForMinorType(MinorType minorType) {
    return originalTypeMap.get(minorType);
  }

  public static DecimalMetadata getDecimalMetadataForField(MaterializedField field) {
    switch(field.getType().getMinorType()) {
      case DECIMAL9:
      case DECIMAL18:
      case DECIMAL28SPARSE:
      case DECIMAL28DENSE:
      case DECIMAL38SPARSE:
      case DECIMAL38DENSE:
      case VARDECIMAL:
        return new DecimalMetadata(field.getPrecision(), field.getScale());
      default:
        return null;
    }
  }

  public static int getLengthForMinorType(MinorType minorType) {
    switch(minorType) {
      case INTERVALDAY:
      case INTERVALYEAR:
      case INTERVAL:
        return 12;
      case DECIMAL28SPARSE:
        return 12;
      case DECIMAL38SPARSE:
      case VARDECIMAL:
        return 16;
      default:
        return 0;
    }
  }

  public static int getMaxPrecisionForPrimitiveType(PrimitiveTypeName type) {
    switch(type) {
      case INT32:
        return 9;
      case INT64:
        return 18;
      case FIXED_LEN_BYTE_ARRAY:
        return DrillRelDataTypeSystem.DRILL_REL_DATATYPE_SYSTEM.getMaxNumericPrecision();
      default:
        throw new UnsupportedOperationException(String.format(
          "Specified PrimitiveTypeName %s cannot be used to determine max precision",
          type));
    }
  }

}
